/* $Id: x86_compat.h,v 1.8 2001/03/25 21:55:56 mikpe Exp $
 * Performance-monitoring counters driver.
 * x86-specific compatibility definitions for 2.2/2.4 kernels.
 *
 * Copyright (C) 2000-2001  Mikael Pettersson
 */
#include <linux/config.h>
#include <linux/version.h>

static __inline__ void write_cr4(unsigned int x)
{
	__asm__ __volatile__("movl %0,%%cr4" : : "r"(x));
}

static __inline__ unsigned int read_cr4(void)
{
	unsigned int x;
	__asm__ __volatile__("movl %%cr4,%0" : "=r"(x));
	return x;
}

static __inline__ void set_in_cr4_local(unsigned int mask)
{
	write_cr4(read_cr4() | mask);
}

static __inline__ void clear_in_cr4_local(unsigned int mask)
{
	write_cr4(read_cr4() & ~mask);
}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)

/* missing from <asm-i386/processor.h> */
#define cpu_has_mmx	(test_bit(X86_FEATURE_MMX,  boot_cpu_data.x86_capability))
#define cpu_has_msr	(test_bit(X86_FEATURE_MSR,  boot_cpu_data.x86_capability))
#define clr_cap_tsc()	(clear_bit(X86_FEATURE_TSC, boot_cpu_data.x86_capability))

#else	/* 2.4 simulation for 2.2 */

#define cpu_has_mmx	(boot_cpu_data.x86_capability & X86_FEATURE_MMX)
#define cpu_has_msr	(boot_cpu_data.x86_capability & X86_FEATURE_MSR)
#define cpu_has_tsc	(boot_cpu_data.x86_capability & X86_FEATURE_TSC)
#define clr_cap_tsc()	(boot_cpu_data.x86_capability &= ~X86_FEATURE_TSC)

#define X86_CR4_TSD	0x0004
#define X86_CR4_PCE	0x0100

unsigned long mmu_cr4_features;	/*fake*/

#define set_in_cr4(mask)	set_in_cr4_local((mask))

#endif	/* 2.4 simulation for 2.2 */

/*
 * Access and export CPU speed.
 * 2.2.xx: cpu_khz when xx >= 16, otherwise cpu_hz
 * 2.4.xx: cpu_khz when xx >= test9-pre, otherwise cpu_hz
 */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,16)
extern unsigned long cpu_khz;
#define EXPORT_cpu_khz	EXPORT_SYMBOL(cpu_khz)
#define get_cpu_khz()	(cpu_khz)
#else
extern unsigned long cpu_hz;
#define EXPORT_cpu_khz	EXPORT_SYMBOL(cpu_hz)
#define get_cpu_khz()	(cpu_hz/1000)
#endif
